@@ -822,12 +822,23 @@ app.controller("MLTaskFeaturesController", function($scope, $controller, $timeou
             && $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon
             && $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon.length > 0;
     };
-    $scope.handleChangeInputMode = function (feature, targetInputMode) {
-        if (feature.role === targetInputMode) {
+
+    $scope.handleChangeInputMode = function (feature, newRole) {
+        if (feature.role === newRole) {
             return;
         }
+        feature.role = newRole;
+        if (feature.role === "REJECT") {
+            $scope.selection.selectedObject = undefined;
+            $scope.selection.selectedObjects = [];
+            return;
+        } else {
+            $scope.selection.selectedObject = feature;
+            $scope.selection.selectedObjects = [feature];
+        }
+
         const oldFeature = angular.copy(feature)
-        feature.role = targetInputMode;
+
         $scope.fixupFeatureConfiguration(feature, oldFeature);
         if ($scope.hasShiftFromHorizon(feature)) {
             CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-reset-shift-modal.html", $scope, "PMLChangeBasicParamsModal", function(newScope) {
@@ -836,11 +847,17 @@ app.controller("MLTaskFeaturesController", function($scope, $controller, $timeou
         }
     };
 
-    $scope.handleMultipleInputChange = function (features, targetInputMode) {
+    $scope.handleMultipleInputChange = function (features, newRole) {
+
+        if ($scope.mlTaskDesign.predictionType !== 'TIMESERIES_FORECAST') {
+            $scope.groupSet({role:newRole})
+            return;
+        }
+
         let shouldOpenModal = false;
-        for (let feature of features.filter(feature => feature.role !== targetInputMode)) {
+        for (let feature of features.filter(feature => feature.role !== newRole)) {
             const oldFeature = angular.copy(feature)
-            feature.role = targetInputMode;
+            feature.role = newRole;
             $scope.fixupFeatureConfiguration(feature, oldFeature);
             if ($scope.hasShiftFromHorizon(feature)) {
                 shouldOpenModal = true;
@@ -853,6 +870,27 @@ app.controller("MLTaskFeaturesController", function($scope, $controller, $timeou
         }
     };
 
+    $scope.handleMultipleTypeChange = function (features, targetType) {
+
+        if ($scope.mlTaskDesign.predictionType !== 'TIMESERIES_FORECAST') {
+            $scope.groupSet({type:targetType})
+            return;
+        }
+
+        let shouldOpenModal = false;
+        for (let feature of features.filter(feature => feature.type !== targetType)) {
+            feature.type = targetType;
+            if ($scope.hasShiftFromHorizon(feature)) {
+                shouldOpenModal = true;
+            }
+        }
+        if (shouldOpenModal) {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-reset-shift-modal.html", $scope, "PMLChangeBasicParamsModal", function(newScope) {
+                newScope.features = features;
+            });
+        }
+    };
+
 });
 
 app.controller("MLTaskOneFeatureController", function($scope, PMLSettings, $stateParams, $filter, DataikuAPI, VisualMlCodeEnvCompatibility, Dialogs) {
@@ -873,6 +911,10 @@ app.controller("MLTaskOneFeatureController", function($scope, PMLSettings, $stat
         if (!$scope.missingValuesModes.some(mode => mode[0] === featureData.missing_handling)) {
             featureData.missing_handling = $scope.missingValuesModes[0][0];
         }
+
+        if (!$scope.featureSupportsAutoShifts(featureData)) {
+            $scope.mlTaskDesign.preprocessing.feature_generation.shifts[featureData._name].from_horizon_mode = 'FIXED';
+        }
     };
 
     $scope.onVariableRoleChange = function() {
@@ -1184,55 +1226,32 @@ app.component("tooManyDroppedRowsWarning", {
         };
     });
 
-    app.filter('formatFeatureType', function() {
-        return function (feature) {
-            if (!feature) {
-                return;
-            }
-            switch (feature.type) {
-                case 'NUMERIC':
-                    return "Numerical";
-                case 'CATEGORY':
-                    return "Categorical";
-                case 'TEXT':
-                    return "Text";
-                case 'VECTOR':
-                    return "Vector";
-                case 'IMAGE':
-                    return "Image";
-            }
-        }
-    });
-
-    app.filter('formatFeatureRole', function() {
-        return function (feature) {
-            if (!feature) {
-                return;
-            }
-            switch (feature.role) {
-                case 'TARGET':
-                    return "Target";
-                case 'INPUT':
-                    return "Known in advance";
-                case 'INPUT_PAST_ONLY':
-                    return "Past only";
-            }
-        }
-    });
-
     app.controller('FeatureGenerationController', function($scope, CreateModalFromTemplate, TimeseriesFeatureGenerationService, TimeseriesForecastingUtils) {
 
         $scope.selectedFeaturesContainsPastCovariate = function () {
-            return $scope.selection.selectedObjects.some(feature => $scope.isPastCovariateFeature(feature._name));
+            return $scope.selection.selectedObjects.some(feature => $scope.isPastOnlyFeature(feature._name));
+        };
+        $scope.allSelectedFeaturesSupportAutoShift = function () {
+            return $scope.selection.selectedObjects.some(feature => !$scope.featureSupportsAutoShifts(feature));
         };
 
-        $scope.setFeatureShifts = function(horizonShifts, forecastShifts) {
+        $scope.setFeatureShifts = function(horizonShifts, forecastShifts, enableHorizonShiftsAuto) {
             $scope.selection.selectedObjects.forEach(function(feature) {
                 if (!$scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name]) {
                     $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name] = {};
                 }
                 $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_forecast = forecastShifts || "";
-                $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon = horizonShifts || "";
+                if (enableHorizonShiftsAuto) {
+                    if ($scope.featureSupportsAutoShifts(feature)) {
+                        $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'AUTO';
+                    } else {
+                        // Do not update horizon shifts for feature not supporting auto
+                    }
+                } else {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon = horizonShifts || "";
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'FIXED';
+                }
+
             });
             $scope.validateShifts();
         };
@@ -1240,40 +1259,119 @@ app.component("tooManyDroppedRowsWarning", {
         $scope.openShiftsModal = function() {
             CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-edit-feature-shift-modal.html", $scope, null, function(newScope) {
                 newScope.save = () => {
-                    $scope.setFeatureShifts(newScope.horizonShifts, newScope.forecastShifts);
+                    $scope.setFeatureShifts(newScope.horizonShifts, newScope.forecastShifts, newScope.enableHorizonShiftsAuto);
                     newScope.dismiss();
                 }
             }, false, false, true)
         };
 
-        $scope.setWindowOperations = function(operations, windowIndex) {
+        $scope.setFeatureAutoShiftsParams = function (maxSelectedHorizonShifts, minHorizonShiftPastOnly, maxHorizonShiftPastOnly, minHorizonShiftKnownInAdvance, maxHorizonShiftKnownInAdvance) {
+            if (!$scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params) {
+                $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params = {};
+            }
+
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_selected_horizon_shifts = maxSelectedHorizonShifts || 1;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_past_only = minHorizonShiftPastOnly;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_past_only = maxHorizonShiftPastOnly;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_known_in_advance = minHorizonShiftKnownInAdvance;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_known_in_advance = maxHorizonShiftKnownInAdvance;
+        };
+
+        $scope.validateAutoShiftsParams = function () {
+            let autoShiftsParams = $scope.mlTaskDesign.preprocessing?.feature_generation?.auto_shifts_params;
+            $scope.hasAutoShiftsParamsValidationErrors = autoShiftsParams === undefined
+                || typeof autoShiftsParams.max_selected_horizon_shifts !== 'number' || autoShiftsParams.max_selected_horizon_shifts < 1
+                || $scope.validatePastOnlyAutoHorizonShiftRange(autoShiftsParams.min_horizon_shift_past_only, autoShiftsParams.max_horizon_shift_past_only) !== ""
+                || $scope.validateKnownInAdvanceAutoHorizonShiftRange(autoShiftsParams.min_horizon_shift_known_in_advance, autoShiftsParams.max_horizon_shift_known_in_advance) !== ""
+        };
+
+        $scope.openAutoShiftsModal = function () {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-edit-auto-shift-modal.html", $scope, null, function (newScope) {
+
+                if ($scope.mlTaskDesign.preprocessing.feature_generation?.auto_shifts_params) {
+                    newScope.maxSelectedHorizonShifts = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_selected_horizon_shifts;
+                    newScope.minHorizonShiftPastOnly = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_past_only;
+                    newScope.maxHorizonShiftPastOnly = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_past_only;
+                    newScope.minHorizonShiftKnownInAdvance = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_known_in_advance;
+                    newScope.maxHorizonShiftKnownInAdvance = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_known_in_advance;
+                } else {
+                    newScope.maxSelectedHorizonShifts = 3 ;
+                    newScope.minHorizonShiftPastOnly = -$scope.mlTaskDesign.predictionLength - 35;
+                    newScope.maxHorizonShiftPastOnly = -$scope.mlTaskDesign.predictionLength;
+                    newScope.minHorizonShiftKnownInAdvance = -35;
+                    newScope.maxHorizonShiftKnownInAdvance = 0;
+                }
+
+                newScope.inputPastOnlyHorizonShiftsRangeValidationError = newScope.validatePastOnlyAutoHorizonShiftRange(newScope.minHorizonShiftPastOnly, newScope.maxHorizonShiftPastOnly);
+                newScope.inputHorizonShiftsRangeValidationError = newScope.validateKnownInAdvanceAutoHorizonShiftRange(newScope.minHorizonShiftKnownInAdvance, newScope.maxHorizonShiftKnownInAdvance);
+
+                newScope.save = () => {
+                    $scope.setFeatureAutoShiftsParams(newScope.maxSelectedHorizonShifts, newScope.minHorizonShiftPastOnly, newScope.maxHorizonShiftPastOnly, newScope.minHorizonShiftKnownInAdvance, newScope.maxHorizonShiftKnownInAdvance);
+                    $scope.validateAutoShiftsParams();
+                    newScope.dismiss();
+                }
+            }, false, false, true)
+        };
+
+        $scope.setWindowOperations = function(numOperations, catOperations, windowIndex) {
             let selectedAndValidFeatures = $scope.uiState.windows[windowIndex].operations_list
                 .filter(el => {
                     return el.$selected && $scope.uiState.windowableFeatures.map(feature => feature._name).includes(el[0]);
                 })
             for (let el of selectedAndValidFeatures) {
-                el[1] = angular.copy(operations)
+                if (el[2].type === "NUMERIC") {
+                    el[1] = angular.copy(numOperations);
+                } else if (el[2].type === "CATEGORY") {
+                    el[1] = angular.copy(catOperations);
+                }
             }
             $scope.validateWindows();
         }
 
-        $scope.openWindowsModal = function(windowIndex, window) {
-            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-edit-feature-windows-modal.html", $scope, null, function(newScope) {
+
+        $scope.openWindowsMassActionModal = function(windowIndex, window) {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-windows-mass-action-modal.html", $scope, null, function(newScope) {
                 let per_feature = $scope.mlTaskDesign.preprocessing.per_feature;
-                let selectedFeatures = window.operations_list.filter(x => x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
-                let someDisabledOperations = new Set(selectedFeatures.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
-                let allEnabledOperations = (new Set(["MEAN", "MEDIAN", "STD", "MIN", "MAX"])).difference(someDisabledOperations);
-                let partiallyEnabledOperations = (new Set(selectedFeatures.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperations);
-                newScope.operations = [
-                    {operation: "MEAN", enabled: allEnabledOperations.has("MEAN"), partiallyEnabled: partiallyEnabledOperations.has("MEAN")},
-                    {operation: "MEDIAN", enabled: allEnabledOperations.has("MEDIAN"), partiallyEnabled: partiallyEnabledOperations.has("MEDIAN")},
-                    {operation: "STD", enabled: allEnabledOperations.has("STD"), partiallyEnabled: partiallyEnabledOperations.has("STD")},
-                    {operation: "MIN", enabled: allEnabledOperations.has("MIN"), partiallyEnabled: partiallyEnabledOperations.has("MIN")},
-                    {operation: "MAX", enabled: allEnabledOperations.has("MAX"), partiallyEnabled: partiallyEnabledOperations.has("MAX")}
+                let selectedFeaturesNumerical = window.operations_list.filter(x => per_feature[x[0]].type === "NUMERIC" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
+                let someDisabledOperationsNumerical = new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
+                let allEnabledOperationsNumerical = (new Set(["MEAN", "MEDIAN", "STD", "MIN", "MAX"])).difference(someDisabledOperationsNumerical);
+                let partiallyEnabledOperationsNumerical = (new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsNumerical);
+                newScope.operationsNumerical = [
+                    {operation: "MEAN", enabled: allEnabledOperationsNumerical.has("MEAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEAN")},
+                    {operation: "MEDIAN", enabled: allEnabledOperationsNumerical.has("MEDIAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEDIAN")},
+                    {operation: "STD", enabled: allEnabledOperationsNumerical.has("STD"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("STD")},
+                    {operation: "MIN", enabled: allEnabledOperationsNumerical.has("MIN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MIN")},
+                    {operation: "MAX", enabled: allEnabledOperationsNumerical.has("MAX"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MAX")}
+                ];
+                let selectedFeaturesCategorical = window.operations_list.filter(x => per_feature[x[0]].type === "CATEGORY" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
+                let someDisabledOperationsCategorical = new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
+                let allEnabledOperationsCategorical = (new Set(["FREQUENCY"])).difference(someDisabledOperationsCategorical);
+                let partiallyEnabledOperationsCategorical = (new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsCategorical);
+                newScope.operationsCategorical = [
+                    {operation: "FREQUENCY", enabled: allEnabledOperationsCategorical.has("FREQUENCY"), partiallyEnabled: partiallyEnabledOperationsCategorical.has("FREQUENCY")},
                 ];
                 newScope.window = window;
+                newScope.atLeastOneNumericalSelected = function () {
+                    return selectedFeaturesNumerical.length > 0;
+                }
+                newScope.atLeastOneCategoricalSelected = function () {
+                    return selectedFeaturesCategorical.length > 0;
+                }
                 newScope.save = () => {
-                    $scope.setWindowOperations(newScope.operations, windowIndex);
+                    $scope.setWindowOperations(newScope.operationsNumerical, newScope.operationsCategorical, windowIndex);
+                    newScope.dismiss();
+                }
+            }, false, false, true)
+        };
+
+        $scope.openWindowsSettingsModal = function() {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-windows-settings-modal.html", $scope, null, function(newScope) {
+                newScope.windows_rescale_numericals = $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals;
+                newScope.windows_max_categories = $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories;
+
+                newScope.save = () => {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals = newScope.windows_rescale_numericals;
+                    $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories = newScope.windows_max_categories;
                     newScope.dismiss();
                 }
             }, false, false, true)
