@@ -7,7 +7,7 @@
      * Was previously located here: src/main/platypus/static/dataiku/js/dashboards/view.js
      */
     app.directive('dashboardPage', function($timeout, CreateModalFromTemplate, AnyLoc, $rootScope, ChartFormattingPaneSections, DetectUtils,
-        $state, $stateParams, executeWithInstantDigest,WT1, DashboardFilters, DashboardUtils, TileUtils, FiltersPanelPosition, FiltersPanelDirection, $location, DashboardPageUtils) {
+        $state, $stateParams, executeWithInstantDigest,WT1, DashboardFilters, DashboardUtils, TileUtils, FiltersPanelPosition, FiltersPanelDirection, $location, DashboardPageUtils, translate) {
         return {
             restrict: 'EA',
             replace: true,
@@ -28,6 +28,7 @@
                 canExportDatasets: '='
             },
             link: function(scope) {
+                scope.translate = translate;
 
                 function goToPage(pageIdx) {
                     if (pageIdx < 0) {
@@ -369,7 +370,7 @@
                 };
 
                 function openMoveCopyTileModal(tile) {
-                    CreateModalFromTemplate('/templates/dashboards/insights/move-copy-tile-modal.html', scope, 'MoveCopyTileModalController', function(newScope) {
+                    CreateModalFromTemplate('/templates/dashboards/insights/move-copy-tile-modal.html', scope, 'MoveCopyTileModalController', function(newScope, translate) {
                         newScope.dashboard = scope.dashboard;
                         newScope.page = scope.page;
                         newScope.tile = tile;
@@ -411,10 +412,19 @@
                     goToPage(DashboardPageUtils.getPreviousPageIndex(scope.dashboard, scope.uiState.currentPageIdx));
                 });
 
+                const unregisterDashboardTitleChangedEvent = $rootScope.$on('dashboardPageTitleChanged', function(_event, newTitle) {
+                    scope.page.displayedTitle = newTitle.displayedTitle;
+                    scope.pageTitle = newTitle.displayedTitle;
+                });
+
+                scope.pageTitle = scope.page && scope.page.displayedTitle ? scope.page.displayedTitle :
+                    scope.translate('DASHBOARD.NAVIGATION_DRAWER.PAGE_INDEX', 'Page ' + (scope.pageIdx + 1), { index: scope.pageIdx + 1 });
+
                 scope.$on('$destroy', function() {
                     Mousetrap.unbind(groupingShortcut);
                     Mousetrap.unbind(nextPageShortcuts);
                     Mousetrap.unbind(previousPageShortcuts);
+                    unregisterDashboardTitleChangedEvent();
                 });
             }
         };
