@@ -484,7 +484,6 @@
                     unwatch();
                 });
 
-                data.insight.$fixedUp = true;
             }
             $scope.insight = data.insight;
             $scope.origInsight = angular.copy($scope.insight);
@@ -627,7 +626,7 @@
                 };
 
                 $scope.explorationFiltersAndSearchQuery = null; // Null means retrieve the saved ones
-                $rootScope.$on('$destroy', $rootScope.$on('shakerTableChangedGlobal', (evt, shaker) => {
+                $scope.$on('$destroy', $rootScope.$on('shakerTableChangedGlobal', (evt, shaker) => {
                     $scope.explorationFiltersAndSearchQuery = {
                         explorationFilters: shaker.explorationFilters,
                         globalSearchQuery: shaker.globalSearchQuery
