@@ -7,6 +7,9 @@
         name: 'Dataset',
         nameForTileParams: 'Dataset table',
         desc: 'Partial or whole datatable',
+        i18nNameKey: 'INSIGHTS.DATASET_TABLE.NAME',
+        i18nTileParamsNameKey: 'INSIGHTS.DATASET_TABLE.TILE_PARAMS_NAME',
+        i18nDescKey: 'INSIGHTS.DATASET_TABLE.DESC',
         icon: 'icon-table',
         color: 'dataset',
 
@@ -242,7 +245,7 @@
                 }
                 $scope.requestedSampleId = null;
 
-                $scope.insight.$filteringStatus = {};
+                $scope.insight.$tileFilteringInformation = {};
                 const initialFilters = angular.copy($scope.shaker.explorationFilters);
 
                 $scope.getDataSpec = () => {
@@ -359,15 +362,15 @@
                 $scope.globalFiltersData = {
                     insightId: $scope.insight.id,
                     applyFilters: executeFiltersRequest,
-                    filteringStatus: angular.copy($scope.insight.$filteringStatus),
+                    tileFilteringInformation: angular.copy($scope.insight.$tileFilteringInformation),
                     sourceDataset: $scope.insight.params.datasetSmartName,
                     sampleSettings: DatasetChartsUtils.makeSelectionFromScript($scope.insight.params.shakerScript),
                     pageId: $stateParams.pageId
                 };
 
-                // globalFiltersData.filteringStatus is mutated by applyFilters, and we need to update insight.$filteringStatus reference to trigger Angular's digest cycle.
-                $scope.$watch('globalFiltersData.filteringStatus', filteringStatus => {
-                    $scope.insight.$filteringStatus = angular.copy(filteringStatus);
+                // globalFiltersData.tileFilteringInformation is mutated by applyFilters, and we need to update insight.$tileFilteringInformation reference to trigger Angular's digest cycle.
+                $scope.$watch('globalFiltersData.tileFilteringInformation', tileFilteringInformation => {
+                    $scope.insight.$tileFilteringInformation = angular.copy(tileFilteringInformation);
                 }, true);
 
                 const registerTableWatch = () => {
@@ -402,7 +405,7 @@
                 });
 
                 $scope.getSamplingSummaryMessage = (clickActionMessage) => {
-                    let samplingMessage = '<i class="dku-icon-warning-fill-16 tooltip-icon"></i> Data sampling can be misrepresentative</br>';
+                    let samplingMessage = `<i class="dku-icon-warning-fill-16 tooltip-icon"></i> ${translate('DASHBOARD.TILES.HEADER.TOOLTIP.DATA_SAMPLING_NOT_REPRESENTATIVE', 'Data sampling can be misrepresentative')}</br>`;
                     if (clickActionMessage) {
                         samplingMessage += `${clickActionMessage}</br>`;
                     }
@@ -418,7 +421,7 @@
                     if (sampleMetadata && !sampleMetadata.sampleIsWholeDataset) {
                         $scope.tile.$sampleMetadata = sampleMetadata;
                         $scope.tile.$samplingSummaryMessage = $scope.getSamplingSummaryMessage();
-                        $scope.tile.$clickableSamplingSummaryMessage = $scope.getSamplingSummaryMessage('Click to edit insight');
+                        $scope.tile.$clickableSamplingSummaryMessage = $scope.getSamplingSummaryMessage(translate('DASHBOARD.TILES.HEADER.TOOLTIP.CLICK_TO_EDIT', 'Click to edit insight'));
                     }
                 };
             }
@@ -451,7 +454,7 @@
         };
     });
 
-    app.directive('datasetTableInsightCreateForm', function(DataikuAPI) {
+    app.directive('datasetTableInsightCreateForm', function(DataikuAPI, translate) {
         return {
             templateUrl: '/templates/dashboards/insights/dataset_table/dataset_table_create_form.html',
             scope: true,
@@ -468,12 +471,18 @@
                         });
                 };
 
-                $scope.hook.defaultName = 'Dataset table';
+                $scope.hook.defaultName = translate('INSIGHTS.DATASET_TABLE.TILE_PARAMS_NAME', 'Dataset table');
                 $scope.$watch('hook.sourceObject', function(nv) {
                     if (!nv || !nv.label) {
                         return;
                     }
-                    $scope.hook.defaultName = nv.label + ' table';
+                    $scope.hook.defaultName = translate(
+                        'INSIGHTS.DATASET_TABLE.NAME_WITH_LABEL',
+                        nv.label + ' table',
+                        {
+                            label: nv.label
+                        }
+                    );
                 });
             }
         };
