@@ -6,7 +6,7 @@
     /**
      * Was previously located here: src/main/platypus/static/dataiku/js/dashboards/edit.js
      */
-    app.factory('TileUtils', function(DashboardUtils, CreateModalFromTemplate, $rootScope, $stateParams, StateUtils) {
+    app.factory('TileUtils', function(DashboardUtils, CreateModalFromTemplate, $rootScope, $stateParams, StateUtils, translate) {
 
         const DEFAULT_BORDER_OPTIONS = {
             color: '#CCCCCC',
@@ -89,6 +89,7 @@
                     newScope.projectKey = $stateParams.projectKey;
                     newScope.objectType = 'DASHBOARD_TILE';
                     newScope.objectId = tile.imageId;
+                    newScope.translate = translate;
                 },
                 'image-uploader-dialog'
             )
