@@ -24,6 +24,10 @@ app.service('FlowGraphSelection', function($rootScope, FlowGraphFolding, Taggabl
     */
     const svc = this;
 
+    // contains all selected nodes, including duplicated objects node if they are in multiple flow-zones
+    // however, when an item is selected multiple times, only one instance of it has the field 'selected' set to true
+    // so calling FlowGraphSelection.getSelectedNodes will only get one instance of each real item
+    // it may however not be the one from its source flow-zone.
     let selectedItems = [];
 
     function _isSelected(item) {
@@ -62,14 +66,13 @@ app.service('FlowGraphSelection', function($rootScope, FlowGraphFolding, Taggabl
     function _addToSelection(items) {
 
         items = items.filter(it => !_isSelected(it));
-        //For nodes used in multiple zones, we need to use a flag to know which one is truely selected
-        items.forEach(it => it.selected = true);
         var realIdsSelected = items.map(it => it.realId);
         selectedItems = selectedItems.filter(it => !it.realId || !realIdsSelected.includes(it.realId));
         if (!items.length) {
             return;
         }
 
+        const addedNodes = [];
         const selector = items.map(it => _isZoned(it) ? `svg [data-node-id="${it.realId}"]` : _isZone(it) ? `svg [id="cluster_${it.id}"]` : `svg [data-id="${it.id}"]`).join(', ');
         d3.selectAll(selector).each(function() {
             const type = this.getAttribute('data-type');
@@ -78,9 +81,15 @@ app.service('FlowGraphSelection', function($rootScope, FlowGraphFolding, Taggabl
                 key = key.replace("cluster_", "");
                 FlowGraphHighlighting.highlightZoneCluster(this)
             }
-            selectedItems.push(FlowGraph.node(key));
+            const node = FlowGraph.node(key);
+            node.selected = false; // we need a single node per realId marked as .selected => step 1: cleanup all nodes
+            addedNodes.push(node);
+            selectedItems.push(node);
         }).classed("selected", true);
 
+        // we need a single node per realId marked as .selected => step 2: add one selected per realId
+        _.uniqBy(addedNodes, it => it.realId).forEach(it => it.selected = true);
+
         if (selectedItems.length) {
             $('#flow-graph').addClass('has-selection');
         }
@@ -285,11 +294,10 @@ app.service('FlowGraphSelection', function($rootScope, FlowGraphFolding, Taggabl
             _clearSelection();
             _addToSelection(nodes);
             svc.recomputeHighlight();
-
-            d3.selectAll('#flow-graph .node:not(.highlight), #flow-graph .edge:not(.highlight)').classed('fade-out', true);
-            $rootScope.$emit('flowSelectionUpdated');
-            $rootScope.$emit('flowDisplayUpdated');
         }
+        d3.selectAll('#flow-graph .node:not(.highlight), #flow-graph .edge:not(.highlight)').classed('fade-out', true);
+        $rootScope.$emit('flowSelectionUpdated');
+        $rootScope.$emit('flowDisplayUpdated');
     }
 
     this.onItemClick = function (item, evt) {
