@@ -184,7 +184,7 @@ const app = angular.module('dataiku.common.nav');
                 const isDataset = nodeType => nodeType === 'LOCAL_DATASET' || nodeType === 'FOREIGN_DATASET';
                 const isManagedFolder = nodeType => nodeType === 'LOCAL_MANAGED_FOLDER' || nodeType === 'FOREIGN_MANAGED_FOLDER';
                 const isSavedModel = nodeType => nodeType === 'LOCAL_SAVEDMODEL' || nodeType === 'FOREIGN_SAVEDMODEL';
-                const isModelEvaluationStore = nodeType => nodeType === 'LOCAL_MODELEVALUATIONSTORE' || nodeType === 'FOREIGN_MODELEVALUATIONSTORE';
+                const isModelEvaluationStore = nodeType => nodeType === 'LOCAL_MODELEVALUATIONSTORE' || nodeType === 'FOREIGN_MODELEVALUATIONSTORE' || nodeType === 'LOCAL_GENAIEVALUATIONSTORE' || nodeType === 'FOREIGN_GENAIEVALUATIONSTORE';
                 const isRetrievableKnowledge = nodeType => nodeType === 'LOCAL_RETRIEVABLE_KNOWLEDGE' || nodeType === 'FOREIGN_RETRIEVABLE_KNOWLEDGE';
                 const isInsight = nodeType => nodeType === 'INSIGHT';
                 const isSavedModelOrModelEvaluationStoreOrInsight = nodeType => isSavedModel(nodeType) || isModelEvaluationStore(nodeType) || isInsight(nodeType);
@@ -676,7 +676,7 @@ const app = angular.module('dataiku.common.nav');
                             }
                         });
 
-                        enteringCompEls.filter('g[data-type="LOCAL_SAVEDMODEL"], g[data-type="FOREIGN_SAVEDMODEL"], g[data-type="LOCAL_MODELEVALUATIONSTORE"], g[data-type="FOREIGN_MODELEVALUATIONSTORE"]').selectAll('rect')
+                        enteringCompEls.filter('g[data-type="LOCAL_SAVEDMODEL"], g[data-type="FOREIGN_SAVEDMODEL"], g[data-type="LOCAL_MODELEVALUATIONSTORE"], g[data-type="FOREIGN_MODELEVALUATIONSTORE"], g[data-type="LOCAL_GENAIEVALUATIONSTORE"], g[data-type="FOREIGN_GENAIEVALUATIONSTORE"]').selectAll('rect')
                             .attr('transform', 'rotate(45) scale(0.7071) scale(1.05)');
 
                         // Create runnables
@@ -993,6 +993,8 @@ const app = angular.module('dataiku.common.nav');
                 }
                 case 'LOCAL_MODELEVALUATIONSTORE':
                 case 'FOREIGN_MODELEVALUATIONSTORE':
+                case 'LOCAL_GENAIEVALUATIONSTORE':
+                case 'FOREIGN_GENAIEVALUATIONSTORE':
                     return "icon-model-evaluation-store";
                 case 'LOCAL_RETRIEVABLE_KNOWLEDGE':
                 case 'FOREIGN_RETRIEVABLE_KNOWLEDGE':
@@ -1033,6 +1035,8 @@ const app = angular.module('dataiku.common.nav');
             "FOREIGN_RETRIEVABLE_KNOWLEDGE": "RETRIEVABLE_KNOWLEDGE",
             "LOCAL_MODELEVALUATIONSTORE": "MODEL_EVALUATION_STORE",
             "FOREIGN_MODELEVALUATIONSTORE": "MODEL_EVALUATION_STORE",
+            "LOCAL_GENAIEVALUATIONSTORE": "GENAI_EVALUATION_STORE",
+            "FOREIGN_GENAIEVALUATIONSTORE": "GENAI_EVALUATION_STORE",
             "LOCAL_DATASET": "DATASET",
             "FOREIGN_DATASET": "DATASET",
             "LOCAL_MANAGED_FOLDER": "MANAGED_FOLDER",
