@@ -196,15 +196,16 @@ app.controller('IPythonController', function ($scope, $stateParams, $state, $sce
         }
     };
 
-    Notification.registerEvent("logout", function(_, message) {
+    const logoutNotificationUnsubscribe = Notification.registerEvent("logout", function(_, message) {
         if (message.user === $scope.appConfig.login) {
             DataikuAPI.profile.get().error(() => {
                 $scope.loggedIn = false;
             });
         }
     });
+    $scope.$on('$destroy', logoutNotificationUnsubscribe);
 
-    Notification.registerEvent("login", function(_, message) {
+    const loginNotificationUnsubscribe = Notification.registerEvent("login", function(_, message) {
         if (message.user === $scope.appConfig.login) {
             DataikuAPI.profile.get().success(() => {
                 $scope.loggedIn = true;
@@ -212,14 +213,16 @@ app.controller('IPythonController', function ($scope, $stateParams, $state, $sce
             });
         }
     });
+    $scope.$on('$destroy', loginNotificationUnsubscribe);
 
-    Notification.registerEvent("session-expiration", (_, message) => {
+    const sessionExpirationNotificationUnsubscribe = Notification.registerEvent("session-expiration", (_, message) => {
         if (message.user === $scope.appConfig.login) {
             DataikuAPI.profile.get().error(() => {
                 $scope.sessionExpired = true;
             });
         }
     });
+    $scope.$on('$destroy', sessionExpirationNotificationUnsubscribe);
 
     $scope.forceReloadJupyterNotebook = function() {
 
