@@ -8,7 +8,7 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             Assert, DataikuAPI, WT1, TopNav, Breadcrumb, CreateModalFromTemplate, TaggingService, FlowGraphSelection,
             GlobalProjectActions, Notification, HistoryService, FutureWatcher, ProgressStackMessageBuilder, ProjectActionsService,
             AlationCatalogChooserService, WebAppsService, DatasetUtils, FlowGraph, TaggableObjectsService, ComputableSchemaRecipeSave,
-            TypeMappingService, TYPE_MAPPING, Logger, Ng1SamplesService) {
+            TypeMappingService, TYPE_MAPPING, Logger, Ng1SamplesService, LastOpenedHistoryService) {
     $controller('_CreateRecipesBehavior', {$scope: $scope});
 
     $scope.standardizedSidePanel = {};
@@ -27,6 +27,8 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
         }
     });
 
+    LastOpenedHistoryService.projects.opened($stateParams.projectKey);
+
     const getProjectAccessInfo = () => {
         DataikuAPI.projects.getProjectAccessInfo($stateParams.projectKey).success(function(accessInfo) {
             delete $rootScope.projectSummary;
@@ -56,6 +58,8 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             $scope.projectSummaryStatus = data.objectsCounts;
 
             TopNav.setProjectData($scope.projectSummary, $scope.projectCurrentBranch);
+            HistoryService.recordProjectOpen($scope.projectSummary);
+
             $scope.topNav.isProjectAnalystRO = $scope.isProjectAnalystRO();
             $scope.topNav.isProjectAnalystRW = $scope.isProjectAnalystRW();
             $scope.topNav.isCurrentProjectAdmin = $scope.isProjectAdmin();
@@ -89,6 +93,8 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
 
             $scope.isObjectSharingRequestEnabled = $rootScope.appConfig.objectSharingRequestsMode === 'ALL_ENABLED' ||
                                             ($rootScope.appConfig.objectSharingRequestsMode !== 'ALL_DISABLED' && $scope.projectSummary.sharingRequestsEnabled);
+            
+            $scope.updateDatasetStorageConflictsWarning();
         }).error(setErrorInScope.bind($scope));
     };
 
@@ -222,9 +228,9 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
         function isEligibleRecipe(recipeType) {
             // generate statistics recipes -> completely different schema, could not possibly want to reconnect
             // eval recipes -> makes no sense to reconnect
-            const excluded_categories = [TYPE_MAPPING.RECIPE_CATEGORIES.EVALUATION, TYPE_MAPPING.RECIPE_CATEGORIES.STANDALONE, TYPE_MAPPING.RECIPE_CATEGORIES.EDA];
-            const excluded_types = ['nlp_llm_evaluation'];
-            return !excluded_categories.includes(TypeMappingService.mapRecipeTypeToCategory(recipeType)) && !excluded_types.includes(recipeType);
+            const excludedCategories = [TYPE_MAPPING.RECIPE_CATEGORIES.EVALUATION, TYPE_MAPPING.RECIPE_CATEGORIES.STANDALONE, TYPE_MAPPING.RECIPE_CATEGORIES.EDA];
+            const excludedTypes = ['nlp_llm_evaluation', 'nlp_agent_evaluation'];
+            return !excludedCategories.includes(TypeMappingService.mapRecipeTypeToCategory(recipeType)) && !excludedTypes.includes(recipeType);
         }
 
         function hasSuccessors(dsNode) {
@@ -417,8 +423,18 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
     $scope.newManagedFolder = function() {
         CreateModalFromTemplate("/templates/managedfolder/new-box-modal.html", $scope);
     };
-    $scope.newModelEvaluationStore = function() {
-        CreateModalFromTemplate("/templates/modelevaluationstores/new-model-evaluation-store-modal.html", $scope);
+    $scope.newModelEvaluationStore = function(allowTabular = true, allowGenAi = false) {
+        $scope.allowTabularCreation = allowTabular;
+        $scope.allowGenAiCreation = allowGenAi;
+        if (!$rootScope.appConfig.licensedFeatures.advancedLLMMeshAllowed && allowGenAi && !allowTabular) {
+            CreateModalFromTemplate("/templates/llm/advanced-llm-mesh-required.html", $scope, null, function(modalScope) {
+                modalScope.uiState = {
+                    "forbiddenFeature" : "GenAI Evaluation Store"
+                }
+            });
+        } else {
+            CreateModalFromTemplate("/templates/modelevaluationstores/new-model-evaluation-store-modal.html", $scope);
+        }
     };
     $scope.newSample = function() {
         Ng1SamplesService.openSampleModal($scope.projectSummary.projectKey, $scope.getRelevantZoneId());
@@ -826,6 +842,29 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             $scope.projectGovernanceStatus = { loading: false, error: fatalAPIError };
         });
     };
+
+    /* Dataset table names conflicts warning */
+    $scope.updateDatasetStorageConflictsWarning = () => {
+        if($scope.isProjectAnalystRW()) {
+            return DataikuAPI.flow.testDatasetStorageConflicts($stateParams.projectKey).then(({data}) => {
+                $scope.testDatasetStorageConflicts = data;
+            }).catch(setErrorInScope.bind($scope));
+        } else {
+            $scope.testDatasetStorageConflicts = {
+                totalManagedWithIssue: 0,
+                duplicates: [],
+            };
+            return $q.resolve();
+        }
+    };
+
+    $scope.showTableNameDuplicateDetails = () => {
+        // we update before showing the modal in case user changed something since
+        $scope.updateDatasetStorageConflictsWarning().then(() => {
+            CreateModalFromTemplate('/templates/datasets/modals/duplicate-table-name-flow-modal.html', $scope, null)
+        })
+    };
+
 });
 
 app.controller("MoveProjectModalController", function($scope, $rootScope, $controller, DataikuAPI, $stateParams, translate, ActivityIndicator) {
@@ -932,6 +971,7 @@ app.service('ProjectActionsService', function($rootScope, $stateParams, CreateMo
                 exportInsights: true,
                 exportNotebooksWithOutputs: true,
                 exportEditableDatasets: true,
+                exportDatasetResources: true,
                 exportGitRepository: canExportGitRepository
             };
 
@@ -1005,7 +1045,7 @@ app.service('ProjectActionsService', function($rootScope, $stateParams, CreateMo
     }
 });
 
-app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplate, RecipeDescService, SmartId, DatasetUtils, FeatureFlagsService) {
+app.controller('_CreateRecipesBehavior', function($scope, $stateParams, CreateModalFromTemplate, RecipeDescService, SmartId, DatasetUtils, FeatureFlagsService, SavedModelsService) {
     function preselect(inputDatasetSmartName,zone,recipeAdditionalParams) {
         return function(newScope) {
             newScope.zone = zone;
@@ -1204,6 +1244,17 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
+    $scope.showCreateExtractContentModal = function(folderId, folderProjectKey, folderName, zone) {
+        CreateModalFromTemplate('/templates/recipes/Nto1-recipe-creation.html', $scope, 'ExtractContentRecipeCreationController', function(newScope) {
+            newScope.zone = zone;
+            if (folderId && folderProjectKey && folderName) {
+                let inputFolderSmartName = SmartId.create(folderId, folderProjectKey);
+                newScope.recipe.inputs.main.items.push({ref: inputFolderSmartName});
+                newScope.$broadcast('preselectInputFolder', {smartName: inputFolderSmartName, name: folderName});
+            }
+        });
+    };
+
     $scope.showSQLRecipeModal = function(inputDatasetSmartName, zone, recipeAdditionalParams) {
         CreateModalFromTemplate('/templates/flow-editor/new-sql-recipe-box.html', $scope, null, function(newScope) {
             newScope.zone = zone;
@@ -1231,6 +1282,19 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
+    $scope.getLLMIdFromSelection = function(selection) {
+        if (!SavedModelsService.isLLM(selection)) {
+            return;
+        }
+        const llmId = SmartId.create(selection.interest.objectId, selection.interest.projectKey);
+        if (SavedModelsService.isAgent(selection)) {
+            return "agent:" + llmId;
+        }
+        if (SavedModelsService.isRetrievalAugmentedLLM(selection)) {
+            return "retrieval-augmented-llm:" + llmId;
+        }
+    }
+
     $scope.showCreatePromptModal = function(preselectedInput, preselectedImageFolder, initialPayload, zone) {
         CreateModalFromTemplate('/templates/recipes/prompt-recipe-creation.html', $scope, 'PromptRecipeCreationController', function(newScope) {
             newScope.zone = zone;
@@ -1298,11 +1362,21 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
-    $scope.showCreateNLPLLMEvaluationRecipeModal = function(preselectedInput, initialPayload, zone) {
+    $scope.showTestInPythonNotebookModal = function (preselectedInput) {
+        const kb = preselectedInput[0];
+        $scope.retrievableKnowledge = {
+            id: SmartId.create(kb.id, kb.projectKey),
+            name: kb.displayName,
+        };
+        CreateModalFromTemplate('/templates/retrievable-knowledge/test-in-python-notebook.html', $scope, 'TestInPythonNotebookModalController');
+    };
+
+    $scope.showCreateLLMEvaluationRecipeModal = function(preselectedInput, initialPayload, zone) {
         if ($scope.appConfig.licensedFeatures.advancedLLMMeshAllowed) {
-            CreateModalFromTemplate('/templates/recipes/new-llm-evaluate-recipe.html', $scope, 'NLPLLMEvaluationRecipeCreationController', function(newScope) {
+            CreateModalFromTemplate('/templates/recipes/new-llm-evaluate-recipe.html', $scope, 'LLMEvaluationRecipeCreationController', function(newScope) {
                 newScope.zone = zone;
                 newScope.initialPayload = initialPayload;
+                newScope.mesFlavor = "LLM";
                 if (preselectedInput) {
                     newScope.uiState.inputDs = preselectedInput;
                 }
@@ -1316,6 +1390,25 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         }
     };
 
+    $scope.showCreateAgentEvaluationRecipeModal = function(preselectedInput, initialPayload, zone) {
+        if ($scope.appConfig.licensedFeatures.advancedLLMMeshAllowed) {
+            CreateModalFromTemplate('/templates/recipes/new-agent-evaluate-recipe.html', $scope, 'AgentEvaluationRecipeCreationController', function(newScope) {
+                newScope.zone = zone;
+                newScope.initialPayload = initialPayload;
+                newScope.mesFlavor = "AGENT";
+                if (preselectedInput) {
+                    newScope.uiState.inputDs = preselectedInput;
+                }
+            });
+        } else {
+            CreateModalFromTemplate("/templates/llm/advanced-llm-mesh-required.html", $scope, null, function(modalScope) {
+                modalScope.uiState = {
+                    "forbiddenFeature" : "Agent evaluation"
+                }
+            });
+        }
+    };
+
     $scope.showCreateCustomCodeRecipeModal = function(recipeType, inputRefs, inputRole, zone){
         CreateModalFromTemplate('/templates/recipes/custom-code-recipe-creation.html', $scope, null, function(newScope) {
             newScope.zone = zone;
@@ -1417,7 +1510,6 @@ app.controller('ContextualProjectHomeController', function($scope, $state, $stat
         if (!nv)  return;
         if (nv.projectKey === $stateParams.projectKey) {
             let stateToRedirectTo = 'projects.project.home.summary';
-            HistoryService.recordProjectOpen($scope.projectSummary);
             $timeout(() => ProjectHomeContextService.recordProjectViewed($stateParams.projectKey));
             if (!$stateParams.discussionId) {
                 stateToRedirectTo = ProjectHomeContextService.getProjectRedirectState(nv, $scope.projectSummaryStatus);
@@ -1486,8 +1578,8 @@ app.controller('ProjectHomeController', function($scope, $state, $stateParams, $
     $scope.$watch("projectSummary", function(nv) {
         if (!nv)  return;
         if (nv.projectKey == $stateParams.projectKey) {
+            WT1.event("open-project-summary", { projectAppType: $scope.projectSummary.projectAppType });
             if ($scope.projectSummary.name) {
-                HistoryService.recordProjectOpen($scope.projectSummary);
                 TopNav.setPageTitle($scope.projectSummary.name);
             }
             if ($scope.projectSummary.projectAppType == 'APP_INSTANCE' && nv.projectKey) {
