@@ -126,7 +126,7 @@ app.directive("recipeIoOutputs", function(RecipesUtils, RecipeComputablesService
                 $scope.canAppend = function(computable) {
                     if (computable.noAppend) return false; // no ambiguity here
                     if (['cpython', 'ksql', 'csync', 'streaming_spark_scala'].indexOf($scope.recipe.type) >= 0) return false; // can't overwrite with continuous activities
-                    if (['upsert'].indexOf($scope.recipe.type) >= 0) return false; // append or overwrite is meaningless for upsert
+                    if (['upsert', 'extract_content'].indexOf($scope.recipe.type) >= 0) return false; // append or overwrite is meaningless for those recipes
                     const selectedEngine = $scope.recipeStatus ? $scope.recipeStatus.selectedEngine : null;
                     if (computable.onlyAppendOnStreamOrSQLEngine && selectedEngine) {
                         //`canEngineAppend` is defined by the backend, but for now,
@@ -602,6 +602,7 @@ app.controller("_RecipeOutputNewManagedBehavior", function($scope, WT1, Logger,
 
     $scope.getEvaluationStoreCreationSettings = function() {
         return {
+            mesFlavor: $scope.mesFlavor,
             partitioningOptionId : $scope.newOutputMES.partitioningOption,
             zone: $scope.zone
         };
@@ -768,7 +769,8 @@ app.directive("recipeIoOutputAddList", function($controller, Assert, RecipesUtil
                         return "create";
                     } else if (role.acceptsManagedFolder) {
                         return "new-odb";
-                    } else if (role.acceptsModelEvaluationStore) {
+                    } else if (RecipeComputablesService.acceptsEvaluationStore(role)) {
+                        RecipeComputablesService.setMesFlavor($scope, role);
                         return "new-mes";
                     } else {
                         return "select";
@@ -780,7 +782,8 @@ app.directive("recipeIoOutputAddList", function($controller, Assert, RecipesUtil
                     $scope.io.newOutputTypeRadio = selectOption($scope.role);
                 } else if ( $scope.io.newOutputTypeRadio == "new-odb" && !$scope.role.acceptsManagedFolder ) {
                     $scope.io.newOutputTypeRadio = selectOption($scope.role);
-                } else if ( $scope.io.newOutputTypeRadio == "new-mes" && !$scope.role.acceptsModelEvaluationStore ) {
+                } else if ($scope.io.newOutputTypeRadio == "new-mes" && RecipeComputablesService.acceptsEvaluationStore($scope.role)) {
+                    RecipeComputablesService.setMesFlavor($scope, $scope.role);
                     $scope.io.newOutputTypeRadio = selectOption($scope.role);
                 }
             };
