@@ -19,12 +19,15 @@
         ChartDimension, ChartLabels, ChartRequestComputer, $state, $timeout, PluginsService, Logger, ActivityIndicator,
         ChartTypeChangeHandler, ChartDefinitionChangeHandler, ChartUADimension, _MapCharts, ChartsStaticData, ChartConfigurationCopyPaste, DetectUtils, ChartIconUtils, ChartSetErrorInScope, ChartFeatures, PolygonSources, ChartAxesUtils,
         ChartZoomControlAdapter, ChartDataUtils, WT1, CHART_TYPES, CHART_VARIANTS, ChartStoreFactory, ColorUtils, $rootScope, $stateParams, ChartsAvailableTypes, RegressionTypes, ChartFilterUtils, ChartFormattingPane, ChartFormattingPaneSections,
-        DataGroupingMode, ChartColorSelection, ConditionalFormattingOptions, ChartDataWrapperFactory, DSSVisualizationThemeUtils) {
+        DataGroupingMode, ChartColorSelection, ConditionalFormattingOptions, ChartDataWrapperFactory, DSSVisualizationThemeUtils, ChartDrilldown, ChartHierarchyDimension, ChartHierarchies, DRILL_UP_SOURCE) {
         return {
             restrict: 'AE',
             templateUrl: '/static/dataiku/js/simple_report/directives/chart-configuration/chart-configuration.directive.html',
-            link: function(scope, element) {
-
+            link: function(scope, element, attrs) {
+                // We can't isolate the scope, so we watch the attribute for changes and update the scope variable
+                scope.$watch(attrs['dropdownMaxHeightPx'], function(newVal) {
+                    scope.dropdownMaxHeightPx = newVal;
+                });
                 //needs to be put in the scope so these functions are accessible in the dashboards
                 scope.canHaveZoomControls = ChartFeatures.canHaveZoomControls;
                 scope.canDisplayLegend = ChartFeatures.canDisplayLegend;
@@ -34,16 +37,24 @@
                 scope.isInAnalysis = $state.current.name.indexOf('analysis') != -1;
                 scope.isInPredicted = $state.current.name.indexOf('predicted') != -1;
                 scope.isInInsight = $state.current.name.indexOf('insight') != -1;
+                scope.isInNoteBooks = $state.current.name.indexOf('notebooks') != -1;
+
+                // Notebooks can actually have multiple charts, but you can only configure one at a time.
+                scope.canConfigureMaxOneChart = scope.isInInsight || scope.isInNoteBooks;
+
                 scope.shouldDisplaySamplingStatus = shouldDisplaySamplingStatus;
                 scope.RegressionTypes = RegressionTypes;
                 scope.PolygonSources = PolygonSources;
 
+                scope.hierarchyDimsInChart = [];
+                scope.breadcrumbs = [];
+                scope.breadcrumbTooltips = {};
+
                 scope.hasLegacyBadge = ChartFeatures.hasLegacyBadge;
                 scope.hasBetaBadge = ChartFeatures.hasBetaBadge;
 
                 scope.legacyBadge = {
-                    tooltip: `You are using the legacy engine.
-                    Click to go back to the new one.`
+                    tooltip: translate('CHARTS.HEADER.BADGE.LEGACY.TOOLTIP', 'You are using the legacy engine. Click to go back to the new one.')
                 };
                 scope.editingTitle = false;
 
@@ -58,7 +69,7 @@
                 scope.availableChartTypes = ChartsAvailableTypes.getAvailableChartTypes();
                 scope.chartTypePickerOptions = { ...scope.chartTypePickerOptions, chartTypes: scope.availableChartTypes };
 
-                if (!scope.isInPredicted && !scope.isInAnalysis) {
+                if (!scope.isInPredicted && !scope.isInAnalysis && !scope.isInNoteBooks) {
                     DataikuAPI.explores.listPluginChartDescs($stateParams.projectKey)
                         .success((data) => {
                             scope.webapps = parseWebapps(data);
@@ -121,7 +132,7 @@
                     const readOnly = $state.current.name === 'projects.project.dashboards.insights.insight.view';
                     if (!readOnly && !(section === ChartFormattingPaneSections.MISC && scope.isInPredicted)) {
                         ChartFormattingPane.open(section, true, true);
-                        if (properties && typeof properties === 'object'){
+                        if (properties && typeof properties === 'object') {
                             Object.keys(properties).forEach(propertyName => {
                                 ChartFormattingPane.setData(propertyName, properties[propertyName]);
                             });
@@ -238,10 +249,6 @@
 
                     // STATIC DATA
                     scope.staticData = {};
-                    scope.staticData.multiplotDisplayModes = [
-                        { id: 'column', name: translate('CHARTS.DROPPED_MEASURE.DISPLAY_AS.BAR') },
-                        { id: 'line', name: translate('CHARTS.DROPPED_MEASURE.DISPLAY_AS.LINE') }
-                    ];
 
                     scope.chartTypes = [
                         {
@@ -385,6 +392,12 @@
 
                     scope.ChartUADimension = ChartUADimension;
 
+                    scope.setDimensionSort = (dimension, newSortId) => {
+                        dimension.sort = angular.copy(scope.chart.def.possibleDimensionSorts.find(
+                            sort => sort.sortId === newSortId
+                        ));
+                    };
+
                     scope.acceptStdAggrTooltipMeasure = function(data) {
                         return ChartTypeChangeHandler.stdAggregatedAcceptMeasureWithAlphanumResults(data);
                     };
@@ -482,7 +495,7 @@
                         scope.chartData = ChartDataWrapperFactory.chartTensorDataWrapper(scope.response.result.pivotResponse, axesDef);
                     };
 
-                    scope.allFilteredOut = false;
+                    scope.resetResponseErrorsInScope();
 
                     /*
                      * Wraps scope.getExecutePromise
@@ -505,7 +518,7 @@
                             Logger.info('Chart is OK, executing');
                             scope.execute();
                         } else {
-                            scope.allFilteredOut = false;
+                            scope.resetResponseErrorsInScope();
                             Logger.info('Chart is NOK, not executing', scope.validity);
                         }
                     };
@@ -567,7 +580,7 @@
 
                                 scope.request = request;
                                 scope.response = data;
-                                scope.allFilteredOut = false;
+                                scope.resetResponseErrorsInScope();
                                 scope.onResponse();
 
                             }).error(function(data, status, headers) {
@@ -587,7 +600,8 @@
                             const oldChartDef = angular.copy(scope.chart.def);
                             scope.recomputeAndUpdateData(changedDefinition);
 
-                            if (!angular.equals(oldChartDef, scope.chart.def)) {
+                            const newImportantChangedDefinition = ChartDefinitionChangeHandler.getRecomputeChange(formatDefinition(scope.chart.def), formatDefinition(oldChartDef));
+                            if (newImportantChangedDefinition) {
                                 Logger.info('Data has been modified, not executing --> will execute at next cycle');
                                 return;
                             }
@@ -640,6 +654,11 @@
                         return {};
                     }
 
+                    scope.$watch('reusableDimensions.length', () => {
+                        scope.updateHierarchyMissingColumns();
+                    });
+
+
                     //This watch needs to be set before the scope.chart.def watch
                     scope.$watch('chart.theme', (nv, ov) => {
                         Logger.info('Chart theme change');
@@ -657,6 +676,11 @@
                         if (nv && (!ov || !angular.equals(nv.themePalettes, ov.themePalettes))) {
                             scope.redraw({ updateThumbnail: true });
                         }
+
+                        if (scope.chart && scope.chart.theme && scope.chart.theme.generalFormatting && scope.chart.theme.generalFormatting.fontFamily) {
+                            element.css('--visualization-font-family', scope.chart.theme.generalFormatting.fontFamily);
+                            element.css('--visualization-font-color', scope.chart.theme.generalFormatting.fontColor);
+                        }
                     });
 
                     // thumbnailData changes are handled by a dedicated watcher
@@ -824,6 +848,9 @@
                      * Recompute/Update handlers
                      */
                     scope.recomputeAndUpdateData = function(changedDefinition) {
+                        const { store, id } = ChartStoreFactory.getOrCreate(scope.chart.def.$chartStoreId);
+                        scope.chart.def.$chartStoreId = id;
+
                         ChartTypeChangeHandler.fixupSpec(scope.chart, scope.chart.theme, changedDefinition);
 
                         scope.canHasTooltipMeasures = [
@@ -839,9 +866,10 @@
                             CHART_TYPES.BINNED_XY
                         ].includes(scope.chart.def.type);
 
-                        scope.canAnimate = ChartFeatures.canAnimate(scope.chart.def.type);
-                        scope.canFacet = ChartFeatures.canFacet(scope.chart.def.type, scope.chart.def.webAppType);
+                        scope.canAnimate = ChartFeatures.canAnimate(scope.chart.def.type, scope.chart.def.variant);
+                        scope.canFacet = ChartFeatures.canFacet(scope.chart.def.type, scope.chart.def.webAppType, scope.chart.def.variant);
                         scope.canFilter = ChartFeatures.canFilter(scope.chart.def.type, scope.chart.def.webAppType);
+                        scope.updateHierarchyInfo();
 
                         return;
                     };
@@ -863,14 +891,14 @@
                         const oldChartType = scope.chart.def.type;
                         element.find('.pivot-charts .mainzone').remove(); // avoid flickering
                         Logger.info('Set chart type');
-                        ChartTypeChangeHandler.onChartTypeChange(scope.chart.def, newChartType.type, newChartType.webappType);
+                        ChartTypeChangeHandler.onChartTypeChange(scope.chart.def, newChartType.type, newChartType.variant, newChartType.webappType);
                         Logger.info('AFTER chart type', scope.chart.def);
                         scope.chart.def.type = newChartType.type;
                         scope.chart.def.variant = newChartType.variant;
                         scope.chart.def.webAppType = newChartType.webappType;
                         if (!scope.validity.valid) {
                             // If chart is not ready and we changed, we can apply things related to new chart type
-                            DSSVisualizationThemeUtils.applyToChart({ chart: scope.chart.def, theme:scope.chart.theme });
+                            DSSVisualizationThemeUtils.applyToChart({ chart: scope.chart.def, theme: scope.chart.theme });
                         }
 
                         if (scope.chart.def.$zoomControlInstanceId) {
@@ -1161,19 +1189,67 @@
                     _.set(axisFormatting, key, value);
                     scope.assignToChartDef('yAxesFormatting', yAxesFormatting);
                 };
+
+                scope.headerDrillUp = (dimension) => {
+                    ChartDrilldown.handleDrillUp(scope.chart.def, dimension, DRILL_UP_SOURCE.CHART_HEADER);
+                };
+
+                /**
+                 * Sets values in scope for:
+                 * hierarchiesInChart - hierarchy dimensions used in chart
+                 * hierarchyDimsInChart - current dimensions of the hierarchies used in chart
+                 * hierarchiesMissingColumns - columns missing per hierarchy (and saves them in chartStore)
+                 * breadcrumbs - breadcrumbs for hierarchies used in chart
+                 * breadcrumbTooltips - text version of breadcrumbs displayed as tooltip for the dimensions in the chart header
+                 */
+                scope.updateHierarchyInfo = () => {
+                    scope.hierarchiesInChart = ChartHierarchyDimension.getHierarchyDimensionsForChartType(scope.chart.def) || [];
+                    scope.updateHierarchyMissingColumns();
+                    scope.hierarchyDimsInChart = ChartHierarchyDimension.getHierarchyDimensionsForChartType(scope.chart.def).map(hierarchy => hierarchy.dimensions[hierarchy.level]);
+                    scope.breadcrumbs = ChartDrilldown.getBreadcrumbs(scope.chart.def);
+
+                    scope.breadcrumbTooltips = {};
+                    scope.hierarchyDimsInChart.forEach(dimension => {
+                        scope.breadcrumbTooltips[dimension.hierarchyId] = ChartDrilldown.getBreadcrumbTooltipForHierarchyDim(scope.breadcrumbs, dimension);
+                    });
+                };
+
+                scope.updateHierarchyMissingColumns = () => {
+                    const { store, id } = ChartStoreFactory.getOrCreate(scope.chart.def.$chartStoreId);
+                    scope.chart.def.$chartStoreId = id;
+                    scope.hierarchiesMissingColumns = ChartHierarchies.getHierarchiesMissingColumns(scope.hierarchiesInChart, scope.usableColumns, scope.reusableDimensions);
+                    store.setHierarchyMissingColumns(scope.hierarchiesMissingColumns);
+                };
+
+                scope.canDimensionDrillUp = (dimension) => {
+                    return ChartHierarchyDimension.canDimensionDrillUp(scope.chart.def, dimension);
+                };
+
+                scope.resetResponseErrorsInScope = () => {
+                    scope.allFilteredOut = false;
+                    scope.display0Warning = false;
+                    scope.hasUnusableHierarchy = false;
+                };
             }
         };
 
         function onPivotRequestError(scope) {
             return (data, status, headers) => {
                 scope.response = undefined;
-                scope.allFilteredOut = false;
-                scope.display0Warning = false;
+                scope.resetResponseErrorsInScope();
 
                 if (data.code === 'FILTERED_OUT') {
                     scope.allFilteredOut = true;
                 } else if (data.code === 'CANNOT_DISPLAY_WITH_EMPTY_AXES') {
                     scope.display0Warning = true;
+                } else if ((data.code === 'MISSING_COLUMN' || data.errorType === 'com.dataiku.dip.exceptions.PivotMissingColumnException')
+                    && !_.isEmpty(scope.hierarchiesMissingColumns)) {
+                    const canDisplayChart = scope.hierarchiesInChart.every(hierarchy => ChartHierarchyDimension.hasHierarchyUsableColumns(scope.chart.def.$chartStoreId, hierarchy));
+                    if (canDisplayChart) {
+                        scope.hierarchiesInChart.forEach((hierarchy) => ChartHierarchyDimension.resetHierarchyToUsableDimension(scope.chart.def.$chartStoreId, hierarchy));
+                    } else {
+                        scope.hasUnusableHierarchy = true;
+                    }
                 } else {
                     if (ChartTypeChangeHandler.hasRequestResponseWarning(scope.chart.def, data)) {
                         scope.setValidity(ChartTypeChangeHandler.getRequestResponseWarning(scope.chart.def, data));
