@@ -8,7 +8,7 @@
 
 
     // (!) This service previously was in static/dataiku/js/simple_report/curves/lines.js
-    function LinesUtils(ChartDimension, Fn, ChartAxesUtils, CHART_TYPES, ColorFocusHandler, SVGUtils, ValuesInChartOverlappingStrategy, ValuesInChartPlacementMode, ChartColorUtils) {
+    function LinesUtils(ChartDimension, Fn, ChartAxesUtils, CHART_TYPES, ColorFocusHandler, SVGUtils, ValuesInChartOverlappingStrategy, ValuesInChartPlacementMode, ChartDrilldown, ChartHierarchyDimension, ChartColorUtils) {
 
 
         const svc = {
@@ -49,7 +49,7 @@
                             svc.drawPath(path, wrapper, emptyBinsMode, redraw, chartBase, svc.xCoord, svc.yCoord, chartData, chartDef, xDimension, xLabels, xAxis, leftYAxis, rightYAxis);
                         });
                 } else {
-                    paths.transition().attr('d', Fn.SELF)
+                    paths.interrupt('updateLines').transition('updateLines').attr('d', Fn.SELF)
                         .each('end', function() {
                             const path = d3.select(this);
                             const wrapper = d3.select(this.parentNode.parentNode);
@@ -137,7 +137,8 @@
                 points.exit().remove();
 
                 points
-                    .transition()
+                    .interrupt('movePoints')
+                    .transition('movePoints')
                     .duration(200)
                     .ease('easeOutQuad')
                     .attr('cx', d => svc.xCoord(xDimension, xLabels, xAxis)(d))
@@ -565,12 +566,12 @@
 
             computeContextualMenuCoords: (datum, facetIndex) => ({ x: datum.xCoord, color: datum.color, facet: facetIndex }),
 
-            getContextualMenuTargetElementProperties: (domElement) => {
+            getContextualMenuTargetElementProperties: (domElement, facetIndex) => {
                 if (domElement.classList.contains('point')) {
                     const element = d3.select(domElement);
                     element.attr('tooltip-el', true);
                     const datum = d3.select(domElement).datum();
-                    return { coords: svc.computeContextualMenuCoords(datum) };
+                    return { coords: svc.computeContextualMenuCoords(datum, facetIndex) };
                 }
                 return null;
             },
@@ -582,7 +583,16 @@
 
                 const tooltipOptions = { displayColorTooltip: chartDef.type !== CHART_TYPES.MULTI_COLUMNS_LINES };
                 chartBase.tooltips.addChartTooltipAndHighlightHandlers(g.node(), (domElement) => svc.getTooltipTargetElementProperties(domElement, facetIndex), colorFocusHandler, tooltipOptions);
-                chartBase.contextualMenu.addChartContextualMenuHandler(g.node(), (domElement) => svc.getContextualMenuTargetElementProperties(domElement, facetIndex));
+                const getTargetElementProperties = (domElement) => svc.getContextualMenuTargetElementProperties(domElement, facetIndex);
+                let getCustomActions;
+                if (ChartHierarchyDimension.getCurrentHierarchyLevel(chartDef) > 0) {
+                    const anchor = SVGUtils.addPlotAreaContextMenuAnchor(g, chartBase.vizWidth, chartBase.vizHeight);
+                    getCustomActions = (properties) => properties ? [] : ChartDrilldown.getDrillupActions(chartDef);
+                    // we need to attach contextual menu to 2 elements in order to have it on background click, since svg (g) has no background
+                    chartBase.contextualMenu.addChartContextualMenuHandler(anchor.node(), getTargetElementProperties, getCustomActions);
+                }
+                chartBase.contextualMenu.addChartContextualMenuHandler(g.node(), getTargetElementProperties, getCustomActions);
+
             },
 
             drawPath: function(path, wrapper, emptyBinsMode, redraw, chartBase, xCoord, yCoord, chartData, chartDef, xDimension, xLabels, xAxis, leftYAxis, rightYAxis) {
@@ -683,8 +693,10 @@
 
             cleanChart: function(g, chartBase) {
                 const wrappers = $('g.wrapper');
+                const bgAnchor = g.select('.chart-contextual-menu-anchor');
                 chartBase.tooltips.removeChartTooltipHandlers(g.node());
                 chartBase.contextualMenu.removeChartContextualMenuHandler(g.node());
+                !bgAnchor.empty() && chartBase.contextualMenu.removeChartContextualMenuHandler(bgAnchor);
                 wrappers.off();
             },
 
@@ -791,14 +803,13 @@
 
             /**
              * Returns true when we must include empty bins to compute measure extent
-             * @param {ChartDef} chartDef
+             * @param {DimensionDef} dimension
              * @return {boolean} true when we must include empty bins to compute measure extent
              */
-            hasEmptyBinsToIncludeAsZero: function(chartDef, chartData) {
+            hasEmptyBinsToIncludeAsZero: function(dimension, chartData) {
                 // In case of a line/mix chart we must look at the emptyBinsMode option
-                return chartDef.genericDimension0.length
-                && chartDef.genericDimension0[0].numParams
-                && chartDef.genericDimension0[0].numParams.emptyBinsMode === 'ZEROS'
+                return dimension.numParams
+                && dimension.numParams.emptyBinsMode === 'ZEROS'
                 && chartData.hasEmptyBins;
             }
         };
