@@ -53,7 +53,7 @@ class RegularEvaluateRecipe(EvaluateRecipe):
                  recipe_desc, script, preparation_output_schema, cond_outputs=None, preprocessing_params=None,
                  evaluation_store_folder=None, evaluation_dataset_type=None, api_node_logs_config=None,
                  diagnostics_folder=None, fmi=None):
-        super(RegularEvaluateRecipe, self).__init__(model_folder, input_dataset_smartname, output_dataset_smartname,
+        super(RegularEvaluateRecipe, self).__init__(model_folder, input_dataset_smartname, managed_folder_smart_id, output_dataset_smartname,
                                                     metrics_dataset_smartname, recipe_desc, script,
                                                     preparation_output_schema,
                                                     cond_outputs, preprocessing_params, evaluation_store_folder,
@@ -74,12 +74,12 @@ class RegularEvaluateRecipe(EvaluateRecipe):
         self.sample_weight = None
         self.modeling_params = None
         self.target_mapping = None
-        self.managed_folder_smart_id = managed_folder_smart_id
 
     def _fetch_input_dataset_and_model_params(self):
         self.input_dataset, self.core_params, self.feature_preproc, self.columns, self.dtypes, self.parse_date_columns = \
-            get_input_parameters(self.model_folder_context, self.input_dataset_smartname, self.preparation_output_schema,
-                                 self.script, self.managed_folder_smart_id)
+            get_input_parameters(self.model_folder_context, self.input_dataset_smartname,
+                                 self.preparation_output_schema, self.script,
+                                 self.managed_folder_smart_id)
         self.prediction_type = self.core_params['prediction_type']
         self.target_column_in_dataset = self.core_params.get("target_variable")
         self.model_target_column = self.core_params.get("target_variable")
