@@ -3,8 +3,9 @@ import numpy as np
 import pandas as pd
 import logging
 
-from dataiku.core.doctor_constants import END_OF_WEEK_DAY, UNIT_ALIGNMENT
+from dataiku.core.doctor_constants import END_OF_WEEK_DAY, UNIT_ALIGNMENT, CATEGORY_HANDLING, DUMMIFY
 from dataiku.core.doctor_constants import NUMBER_OF_TIME_UNITS
+from dataiku.core.doctor_constants import ROLLING_WINDOWS_RESOURCES
 from dataiku.core.doctor_constants import TARGET_VARIABLE
 from dataiku.core.doctor_constants import TIMESERIES_IDENTIFIER_COLUMNS
 from dataiku.core.doctor_constants import TIME_STEP_PARAMS
@@ -100,15 +101,46 @@ def resample_timeseries(
     
     return resampled_df
 
+def should_compute_auto_shifts(preprocessing_params, modeling_params):
+    if not modeling_params.get("isShiftWindowsCompatible", False):
+        return False
+    auto_shift_columns = get_auto_shift_columns(preprocessing_params)
+    has_auto_shift_features = len(auto_shift_columns["past_only"]) > 0 or len(auto_shift_columns["known_in_advance"]) > 0
+    if has_auto_shift_features and not preprocessing_params.get("feature_generation", {}).get("auto_shifts_params", False):
+        raise ValueError("Invalid configuration, some feature generations have auto-shift enabled, but 'auto_shifts_params' were not provided. Under the 'Feature generation' tab, disable auto shifts or set auto shits params, then try again.")
+    else:
+        return has_auto_shift_features
+
+def get_auto_shifts_past_only_range(preprocessing_params):
+    auto_shifts_params = preprocessing_params.get("feature_generation", {}).get("auto_shifts_params", {})
+    return {
+        "min": auto_shifts_params.get("min_horizon_shift_past_only", -1),
+        "max": auto_shifts_params.get("max_horizon_shift_past_only", -1)
+    }
+
+def get_auto_shifts_known_in_advance_range(preprocessing_params):
+    auto_shifts_params = preprocessing_params.get("feature_generation", {}).get("auto_shifts_params", {})
+    return {
+        "min": auto_shifts_params.get("min_horizon_shift_known_in_advance", 0),
+        "max": auto_shifts_params.get("max_horizon_shift_known_in_advance", 0)
+    }
+
 def is_shift_compatible(role):
     return role in ["TARGET", "INPUT", "INPUT_PAST_ONLY"]
 
-def is_window_compatible(feature):
-    return (is_shift_compatible(feature["role"])
-            and feature["type"] == "NUMERIC"
-            and ("numerical_handling" not in feature or feature["numerical_handling"] == "REGULAR"))
+def is_window_num_operation_compatible(feature):
+    return feature["type"] == "NUMERIC" and ("numerical_handling" not in feature or feature["numerical_handling"] == "REGULAR")
 
-def is_past_covariate(role):
+def is_window_cat_operation_compatible(feature):
+    return feature["type"] == "CATEGORY" and ("category_handling" not in feature or feature["category_handling"] == "DUMMIFY")
+
+def is_window_compatible(feature, operation):
+    return (is_shift_compatible(feature["role"]) and (
+            (operation in ["MEAN", "MEDIAN", "STD", "MIN", "MAX"] and is_window_num_operation_compatible(feature)) or
+            (operation == "FREQUENCY" and is_window_cat_operation_compatible(feature))
+    ))
+
+def is_past_only(role):
     return role in ["TARGET", "INPUT_PAST_ONLY"]
 
 
@@ -142,10 +174,10 @@ class ShiftMap(object):
         if self.generated_features_mappings is None or timeseries_identifier not in self.generated_features_mappings:
             return []
         generated_features_mapping = self.generated_features_mappings[timeseries_identifier]
-        preprocessed_columns = generated_features_mapping.get_features_from_original_column(column)
+        preprocessed_columns = generated_features_mapping.get_features_from_origin_column(column)
         return preprocessed_columns
 
-    def is_past_covariate(self, column):
+    def is_past_only(self, column):
         return self.input_shift_map[column]["past_covariate"]
 
     def is_empty(self):
@@ -156,6 +188,22 @@ class ShiftMap(object):
         return not has_at_least_one_shift
 
 
+def get_auto_shift_columns(preprocessing_params):
+    auto_shift_columns = {
+        "past_only": [], # Includes the role TARGET
+        "known_in_advance": []
+    }
+    for column in list(preprocessing_params.get("feature_generation", {}).get("shifts", {}).keys()):
+        shift_params = preprocessing_params["feature_generation"]["shifts"][column]
+        role = preprocessing_params["per_feature"][column]["role"]
+        if is_shift_compatible(role) and shift_params.get("from_horizon_mode", 'FIXED') == 'AUTO':
+            if is_past_only(role):
+                auto_shift_columns["past_only"].append(column)
+            else:
+                auto_shift_columns["known_in_advance"].append(column)
+    return auto_shift_columns
+
+
 def get_shift_map(preprocessing_params, generated_features_mappings):
     """
     :param preprocessing_params: dict
@@ -164,12 +212,14 @@ def get_shift_map(preprocessing_params, generated_features_mappings):
     """
     shift_map = copy.deepcopy(preprocessing_params["feature_generation"]["shifts"])
     for column in list(shift_map.keys()):
-        shift_params = shift_map[column]
         role = preprocessing_params["per_feature"][column]["role"]
         if not is_shift_compatible(role):
             shift_map.pop(column)
         else:
-            shift_params["past_covariate"] = is_past_covariate(role)
+            shift_map[column]["past_covariate"] = is_past_only(role)
+            if shift_map[column].get("from_horizon_mode", 'FIXED') == 'AUTO':
+                # Replace provided values, with automatically selected values
+                shift_map[column]["from_horizon"] = shift_map[column]["from_horizon_auto"]
     for window in get_windows_list(preprocessing_params):
         operations_map = window["operations_map"]
         shift = window["shift"]
@@ -180,17 +230,15 @@ def get_shift_map(preprocessing_params, generated_features_mappings):
                 preprocessed_feature_name = "rolling_window:{}:{}:{}".format(window["length"], operation, feature)
                 if preprocessed_feature_name not in shift_map:
                     shift_params = {
-                        "past_covariate": is_past_covariate(role),
+                        "past_covariate": is_past_only(role),
                         "from_forecast": [],
-                        "from_horizon": []
+                        "from_horizon": [],
                     }
                     shift_map[preprocessed_feature_name] = shift_params
-                else:
-                    shift_params = shift_map[preprocessed_feature_name]
                 if is_from_forecast:
-                    shift_params["from_forecast"].append(shift)
+                    shift_map[preprocessed_feature_name]["from_forecast"].append(shift)
                 else:
-                    shift_params["from_horizon"].append(shift)
+                    shift_map[preprocessed_feature_name]["from_horizon"].append(shift)
     return ShiftMap(shift_map, generated_features_mappings)
 
 
@@ -199,13 +247,17 @@ def get_windows_list(preprocessing_params):
     for window in windows_list:
         operations_map = window["operations_map"]
         for feature, operations in operations_map.items():
-            operations_map[feature] = [op["operation"] for op in operations if op["enabled"] and is_window_compatible(preprocessing_params["per_feature"][feature])]
+            operations_map[feature] = [op["operation"] for op in operations if op["enabled"] and is_window_compatible(preprocessing_params["per_feature"][feature], op["operation"])]
     return windows_list
 
 
 class TimeseriesPreprocessing:
-    def __init__(self, data_folder_context, core_params, preprocessing_params, modeling_params, listener):
+    def __init__(self, data_folder_context, core_params, preprocessing_params, modeling_params, listener, windows_resources_folder_context=None):
         self.data_folder_context = data_folder_context
+        if windows_resources_folder_context is not None:
+            self.windows_resources_folder_context = windows_resources_folder_context
+        else:
+            self.windows_resources_folder_context = data_folder_context
         self.core_params = core_params
         self.preprocessing_params = preprocessing_params
         self.modeling_params = modeling_params
@@ -318,9 +370,13 @@ class TimeseriesPreprocessing:
                 df_of_timeseries_identifier, self.preprocessing_params
             ).build()
 
-        # run_folder should not exist because single timeseries don't save anything
+        # We separate:
+        # - `windows_resources_folder_context` which always refers to the `preprocessing_folder_context` from
+        # - `data_folder_context` which can either refer to:
+        #   - `preprocessing_folder_context` when writing (e.g. preprocessing on the full df) or reading (e.g. preprocessing before scoring) preprocessing steps resources
+        #   - `model_folder_context` when preprocessing steps resources should only be computed and neither written nor read (e.g. preprocessing before HP search)
         preproc_handler = SingleTimeseriesPreprocessingHandler(
-            self.core_params, self.preprocessing_params, self.modeling_params, self.data_folder_context, collector_data, timeseries_identifier
+            self.core_params, self.preprocessing_params, self.modeling_params, self.data_folder_context, self.windows_resources_folder_context, collector_data, timeseries_identifier
         )
 
         if use_saved_resources:
@@ -409,11 +465,12 @@ class TimeseriesPreprocessing:
 
 
 class SingleTimeseriesPreprocessingHandler(PredictionPreprocessingHandler):
-    def __init__(self, core_params, preprocessing_params, modeling_params, data_folder_context, collector_data=None, timeseries_identifier=None):
+    def __init__(self, core_params, preprocessing_params, modeling_params, data_folder_context, windows_resources_folder_context, collector_data=None, timeseries_identifier=None):
         super(SingleTimeseriesPreprocessingHandler, self).__init__(core_params, preprocessing_params, data_folder_context)
         self.modeling_params = modeling_params
         self.collector_data = collector_data
         self.timeseries_identifier = timeseries_identifier
+        self.windows_resources_folder_context = windows_resources_folder_context
 
     @property
     def time_variable(self):
@@ -423,6 +480,16 @@ class SingleTimeseriesPreprocessingHandler(PredictionPreprocessingHandler):
     def target_map(self, with_target=False):
         return None
 
+    def get_windows_resources_map(self):
+        """
+        Note: the windows resources are computed before the preprocessing, only reading them is required.
+        """
+        if self.windows_resources_folder_context.isfile(ROLLING_WINDOWS_RESOURCES):
+            full_windows_resources_map = self.windows_resources_folder_context.read_json(ROLLING_WINDOWS_RESOURCES)
+            return full_windows_resources_map[self.timeseries_identifier]
+        else:
+            return None
+
     def get_collector_data(self):
         return self.collector_data
 
@@ -435,7 +502,7 @@ class SingleTimeseriesPreprocessingHandler(PredictionPreprocessingHandler):
             yield OutputRawColumns(self.core_params[TIMESERIES_IDENTIFIER_COLUMNS], TIMESERIES_IDENTIFIER_COLUMNS)
 
         if self.modeling_params.get("isShiftWindowsCompatible", False):
-            yield FetchRollingWindows(get_windows_list(self.preprocessing_params))
+            yield FetchRollingWindows(get_windows_list(self.preprocessing_params), self.get_windows_resources_map())
 
         for step in super(SingleTimeseriesPreprocessingHandler, self).preprocessing_steps(with_target=True):
             yield step
@@ -452,13 +519,33 @@ class SingleTimeseriesPreprocessingHandler(PredictionPreprocessingHandler):
                 self.set_resource(resource_name, {}, resource_type)
 
 
-def add_rolling_windows(full_df, core_params, windows_list):
+def add_rolling_windows_for_training(full_df, core_params, windows_list, preprocessing_params, folder_context):
+    """"
+    :param folder_context: Required to write the mapping column -> values for categorical-like features
+    """
+    return add_rolling_windows(full_df, core_params, windows_list, preprocessing_params, folder_context, True)
+
+def add_rolling_windows_for_scoring(full_df, core_params, windows_list, preprocessing_params, folder_context):
+    """
+    :param folder_context: Required to read the mapping column -> values for categorical-like features
+    """
+    return add_rolling_windows(full_df, core_params, windows_list, preprocessing_params, folder_context, False)
+
+
+def add_rolling_windows(full_df, core_params, windows_list, preprocessing_params, folder_context, is_training):
     if len(windows_list)==0:
         return full_df
     to_concat = []
+    rescale_numericals = preprocessing_params.get("feature_generation", {}).get("windows_rescale_numericals", False)
+    max_categories = preprocessing_params.get("feature_generation", {}).get("windows_max_categories", 10)
+    windows_res_map = {}
+    if not is_training and folder_context.isfile(ROLLING_WINDOWS_RESOURCES):
+        windows_res_map = folder_context.read_json(ROLLING_WINDOWS_RESOURCES)
     for timeseries_identifier, df_of_timeseries_identifier in timeseries_iterator(
             full_df, core_params[TIMESERIES_IDENTIFIER_COLUMNS]
     ):
+        if is_training:
+            windows_res_map[timeseries_identifier] = {}
         for window in windows_list:
             length = window["length"]
             operations_map = window["operations_map"]
@@ -468,22 +555,65 @@ def add_rolling_windows(full_df, core_params, windows_list):
                 if len(operations_map[column_name]) == 0:
                     continue
                 single_col_series = df_of_timeseries_identifier[column_name].copy()
-                w = single_col_series.rolling(length)
-                for operation in operations_map[column_name]:
-                    window_name = "rolling_window:{}:{}:{}".format(length, operation, column_name)
-                    if window_name not in df_of_timeseries_identifier.columns:
-                        if operation == "MEAN":
-                            result = w.mean()
-                        if operation == "MEDIAN":
-                            result = w.median()
-                        if operation == "STD":
-                            result = w.std()
-                        if operation == "MIN":
-                            result = w.min()
-                        if operation == "MAX":
-                            result = w.max()
-                        df_of_timeseries_identifier["rolling_window:{}:{}:{}".format(length, operation, column_name)] = result
+                feature_type = preprocessing_params["per_feature"][column_name]["type"]
+                if feature_type == "CATEGORY" and preprocessing_params["per_feature"][column_name][CATEGORY_HANDLING] == DUMMIFY:
+                    if is_training:
+                        all_categories, counts = np.unique(single_col_series.values, return_counts=True)
+                        categories = [category for category, _ in sorted(zip(all_categories, counts), key=lambda x: x[1], reverse=True)[:max_categories]]
+                        windows_res_map[timeseries_identifier][column_name] = {}
+                        windows_res_map[timeseries_identifier][column_name]["categories"] = categories
+                    else:
+                        # scoring (recipe or API)
+                        categories = windows_res_map[timeseries_identifier][column_name]["categories"]
+                    other_cat_freq = np.ones_like(single_col_series.values, dtype=float)
+                    for category in categories:
+                        # operation = "FREQUENCY"
+                        result = (single_col_series == category).rolling(length).mean()
+                        other_cat_freq -= result
+                        window_name = "rolling_window:{}:{}:{}:{}".format(length, "FREQUENCY", column_name, category)
+                        df_of_timeseries_identifier[window_name] = result
+                    other_cat_window_name = "rolling_window:{}:{}:{}:{}".format(length, "FREQUENCY", column_name, "__DKU_OTHER__")
+                    df_of_timeseries_identifier[other_cat_window_name] = other_cat_freq
+                elif feature_type == "NUMERIC":
+                    w = single_col_series.rolling(length)
+                    for operation in operations_map[column_name]:
+                        window_name = "rolling_window:{}:{}:{}".format(length, operation, column_name)
+                        if window_name not in df_of_timeseries_identifier.columns:
+                            if operation == "MEAN":
+                                result = w.mean()
+                            elif operation == "MEDIAN":
+                                result = w.median()
+                            elif operation == "STD":
+                                result = w.std()
+                            elif operation == "MIN":
+                                result = w.min()
+                            elif operation == "MAX":
+                                result = w.max()
+                            else:
+                                continue
+                            if rescale_numericals:
+                                if is_training:
+                                    windows_res_map[timeseries_identifier][window_name] = {}
+                                    average = result.mean()
+                                    scale = result.std()
+                                    # Numerical noise occurs in the windows computation so a `result` Series that should be
+                                    # identically of the same value can end up having a small non-zero variance.
+                                    # Considering this, we aggressively consider small `scale` values (relative to `average`) as 0.
+                                    if (average != 0 and abs(scale / average) < 1e-8) or scale == 0.:
+                                        # `scale` is 0: we cannot perform rescaling, so we pick 1 (no rescaling)
+                                        scale = 1.
+                                    windows_res_map[timeseries_identifier][window_name]["average"] = float(average)
+                                    windows_res_map[timeseries_identifier][window_name]["scale"] = float(scale)
+                                else:
+                                    average = windows_res_map.get(timeseries_identifier, {}).get(window_name, {}).get("average", 0.)
+                                    scale = windows_res_map.get(timeseries_identifier, {}).get(window_name, {}).get("scale", 1.)
+                                result = (result - average) / scale
+                            df_of_timeseries_identifier[window_name] = result
+                else:
+                    logger.warning("Unsupported input feature type: " + feature_type)
         to_concat.append(df_of_timeseries_identifier)
+    if is_training:
+        folder_context.write_json(ROLLING_WINDOWS_RESOURCES, windows_res_map)
     return pd.concat(to_concat)
 
 
@@ -497,6 +627,10 @@ class SingleRow(object):
         return pd.DataFrame(np.array([[v] for v in self.feature_values]).T, columns=self.feature_names)
 
 
+def get_rolling_windows_features_from_cat_windows_column_prefix(external_features_columns, column):
+    return [preprocessed_column for preprocessed_column in external_features_columns
+            if preprocessed_column.startswith(column + ":")]
+
 class MultiHorizonShiftExpanderMixin:
     shift_map = None
     target_variable = None
@@ -510,12 +644,13 @@ class MultiHorizonShiftExpanderMixin:
                 out_series = preprocessed_series.shift(-forecast_shift)
                 out_series.name = series_name
                 series_list.append(out_series)
-        for horizon_shift in self.shift_map.get_horizon_shifts(column):
+        horizon_shifts = self.shift_map.get_horizon_shifts(column)
+        for horizon_shift in horizon_shifts:
             forecast_shift = current_horizon + horizon_shift
             if forecast_shift in forecast_shifts:
                 # Prevent duplicate data
                 continue
-            if self.shift_map.is_past_covariate(column) and forecast_shift > 0:
+            if self.shift_map.is_past_only(column) and forecast_shift > 0:
                 logger.info("Ignoring shift from horizon {} looking into the future at horizon {} for past-only feature {}".format(horizon_shift, current_horizon, column))
             else:
                 series_name = "horizon_shift:{}:{}".format(horizon_shift, preprocessed_column)
@@ -524,10 +659,10 @@ class MultiHorizonShiftExpanderMixin:
                     out_series.name = series_name
                     series_list.append(out_series)
 
-    def expand_shifts_for_training(self, external_features_values, target_values, current_horizon, timeseries_identifier):
+    def expand_shifts_for_training(self, df_external_features_values, target_values, current_horizon, timeseries_identifier):
         """
         Note that underlying time value is sorted in increasing order
-        :param external_features_values: pd.DataFrame
+        :param df_external_features_values: pd.DataFrame
         :param target_values: pd.Series
         :param current_horizon: int in the [1, ..., prediction_length] interval, prediction_length being the full prediction horizon
         :param timeseries_identifier: str
@@ -540,16 +675,21 @@ class MultiHorizonShiftExpanderMixin:
             if column == self.target_variable:
                 series = target_values
                 self._update_series_list(column, column, series, current_horizon, series_list)
-            elif external_features_values is not None and column in external_features_values.columns:
+            elif df_external_features_values is not None and column in df_external_features_values.columns:
                 # `column` (the input column) matches a column in the preprocessed dataframe `external_features_values` (e.g. numerical column with standard handling)
-                series = external_features_values[column]
+                series = df_external_features_values[column]
                 self._update_series_list(column, column, series, current_horizon, series_list)
-            else:
+            elif len(self.shift_map.get_preprocessed_columns(column, timeseries_identifier)) > 0:
                 # `column` (the input column) does not match the output columns in the preprocessed dataframe `external_features_values`,
                 # because it has been preprocessed (e.g. dummies of categorical variables), so we need to find the all the output columns
                 # that are computed from `column` in the preprocessing, and apply `column` shift_map settings to them
                 for preprocessed_column in self.shift_map.get_preprocessed_columns(column, timeseries_identifier):
-                    preprocessed_series = external_features_values[preprocessed_column]
+                    preprocessed_series = df_external_features_values[preprocessed_column]
+                    self._update_series_list(column, preprocessed_column, preprocessed_series, current_horizon, series_list)
+            else:
+                # rolling windows for categorical variables
+                for preprocessed_column in get_rolling_windows_features_from_cat_windows_column_prefix(df_external_features_values.columns, column):
+                    preprocessed_series = df_external_features_values[preprocessed_column]
                     self._update_series_list(column, preprocessed_column, preprocessed_series, current_horizon, series_list)
 
         y = target_values[current_horizon:].values  # [y_horizon, y_horizon+1, ..., y_t+horizon] with t+horizon the last available date with value for target
@@ -575,7 +715,7 @@ class MultiHorizonShiftExpanderMixin:
             if forecast_shift in forecast_shifts:
                 # Prevent duplicate data
                 continue
-            if forecast_shift > 0 and self.shift_map.is_past_covariate(column):
+            if forecast_shift > 0 and self.shift_map.is_past_only(column):
                 logger.info("Ignoring shift from horizon {} looking into the future at horizon {} for past-only feature {}".format(horizon_shift, current_horizon, column))
             else:
                 feature_name = "horizon_shift:{}:{}".format(horizon_shift, preprocessed_column)
@@ -609,33 +749,15 @@ class MultiHorizonShiftExpanderMixin:
                 past_series = past_df[column]
                 future_series = future_df[column] if future_df is not None else None
                 self._update_row(column, column, current_horizon, past_series, future_series, row)
-            else:
+            elif len(self.shift_map.get_preprocessed_columns(column, timeseries_identifier)) > 0:
                 for preprocessed_column in self.shift_map.get_preprocessed_columns(column, timeseries_identifier):
                     past_series = past_df[preprocessed_column]
                     future_series = future_df[preprocessed_column] if future_df is not None else None
                     self._update_row(column, preprocessed_column, current_horizon, past_series, future_series, row)
+            else:
+                for preprocessed_column in get_rolling_windows_features_from_cat_windows_column_prefix(past_df.columns, column):
+                    past_series = past_df[preprocessed_column]
+                    future_series = future_df[preprocessed_column] if future_df is not None else None
+                    self._update_row(column, preprocessed_column, current_horizon, past_series, future_series, row)
+
         return row.to_dataframe()
-
-
-def sanitize_feature_names(df):
-    """Sanitizes DataFrame column names for compatibility with all algos.
-    This function replaces special characters in column names that are forbidden by some algos.
-    Each character is replaced with a specific text representation.
-
-    Args:
-        df (pd.DataFrame): The input DataFrame with potentially incompatible column names. It is updated by the method.
-
-    Returns:
-        pd.DataFrame: The DataFrame with sanitized column names.
-    """
-    characters_replacements = {
-        # Forbidden by XGboost
-        '[': '$$open_bracket$$',
-        ']': '$$close_bracket$$',
-        '<': '$$smaller_than$$',
-    }
-    new_columns = df.columns.astype(str)
-    for old_char, new_name in characters_replacements.items():
-        new_columns = new_columns.str.replace(old_char, new_name, regex=False)
-    df.columns = new_columns
-    return df
\ No newline at end of file
